#!/usr/bin/env nix-shell
#!nix-shell -i python -p "python3.withPackages(ps : with ps; [ pyyaml paho-mqtt prompt-toolkit ])"


import os
import sys
import time

from threading import Lock
from threading import Thread


from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.shortcuts import progress_dialog


class DeviceScanDiolog:  # (Thread):

  _pct = 0
  _locks = []

  def __init__(self, timeout: float = 1, fps: int = 30):
    '''timeout ~~> seconds'''

    def _nothing(*args, **kwargs): pass
    self._logTxtCB = _nothing

    self._fps     = fps
    self._timeout = timeout

    self._dialog = progress_dialog (
      title='Searching for devices…',
      text='waiting for devices to appear',
      run_callback=self._progressWorker,
    )


  def addDevice(self, device: str) -> None:
    self._pct = 0
    self._logTxtCB(f'{device}\n')

  def start(self):
    self._dialog.run()


  def _progressWorker(self, setPct, logTxt):

    self._logTxtCB = logTxt

    while not self.isAtEnd():
      self += ((1 / self._fps) / self._timeout) * 100
      setPct(self._pct)
      time.sleep(1 / self._fps)

    time.sleep(1)

#      for lock in self._locks:
#        lock.release()



  def waitUntilFinished(self) -> None:
    lock = Lock()
    self._locks += lock

  def isAtEnd(self) -> bool:
    return self._pct >= 99

  def __add__(self, other: float):
    return self._pct + other

  def __iadd__(self, other: float):
    self._pct += other
    return self

  def __int__(self) -> int:
    return int(self._pct)

  def __float__(self) -> float:
    return float(self._pct)

  def __complex__(self) -> complex:
    return complex(self._pct)




def worker(set_percentage, log_text):
    '''
    This worker function is called by `progress_dialog`. It will run in a
    background thread.

    The `set_percentage` function can be used to update the progress bar, while
    the `log_text` function can be used to log text in the logging window.
    '''
    percentage = 0
    for dirpath, dirnames, filenames in os.walk(sys.argv[1]):
        for f in filenames:
            log_text(f'{dirpath} / {f}\n')
            set_percentage(percentage + 1)
            percentage += 0.1
            time.sleep(0.01)

            if percentage == 100:
                break
        if percentage == 100:
            break

    # Show 100% for a second, before quitting.
    set_percentage(100)
    time.sleep(1)


bindings = KeyBindings()
@bindings.add('c-c')
def exit(event):
  import sys
  print('<exit>')
  sys.exit(0)


if __name__ == '__main__':

  devs = DeviceScanDiolog()

  def _thread(_d=30):
    _tplus = 0
    while _tplus + 0.37 < 1:

      dev = f'AnswerMachine[{_d}]'
      _d += 1
      devs.addDevice(dev)

      time.sleep(0.29 + _tplus)

      dev = f'MyLEETdevice[{_d+1000}]'
      devs.addDevice(dev)

      time.sleep(0.37 + _tplus)
      _tplus += 0.07


  t = Thread(target=_thread)
  t.start()


  devs.start()
  t.join()





